@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008-2013 Yubico AB
+ * Copyright (c) 2008-2014 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,6 +41,10 @@
 #define Sleep(x) usleep((x)*1000)
 #endif
 
+#ifdef YK_DEBUG
+static void _yk_hexdump(void *, int);
+#endif
+
 /*
  * Yubikey low-level interface section 2.4 (Report arbitration polling) specifies
  * a 600 ms timeout for a Yubikey to process something written to it.
@@ -112,10 +116,12 @@ int yk_check_firmware_version2(YK_STATUS *st)
 		st->versionMinor == 1 ||
 		st->versionMinor == 2 ||
 		st->versionMinor == 3 ||
-		st->versionMinor == 4)) ||
+		st->versionMinor == 4 ||
+		st->versionMinor == 5)) ||
 	      (st->versionMajor == 3 &&
 	       (st->versionMinor == 0 ||
-		st->versionMinor == 1)))) {
+		st->versionMinor == 1 ||
+		st->versionMinor == 2)))) {
 		yk_errno = YK_EFIRMWARE;
 		return 0;
 	}
@@ -398,6 +404,7 @@ static const char *errtext[] = {
 	"operation would block",
 	"invalid command for operation",
 	"expected only one YubiKey but several present",
+	"no data returned from device",
 };
 const char *yk_strerror(int errnum)
 {
@@ -454,7 +461,7 @@ int yk_wait_for_key_status(YK_KEY *yk, uint8_t slot, unsigned int flags,
 {
 	unsigned char data[FEATURE_RPT_SIZE];
 
-	unsigned int sleepval = 10;
+	unsigned int sleepval = 1;
 	unsigned int slept_time = 0;
 	int blocking = 0;
 
@@ -467,10 +474,20 @@ int yk_wait_for_key_status(YK_KEY *yk, uint8_t slot, unsigned int flags,
 	slot = 0;
 
 	while (slept_time < max_time_ms) {
+		Sleep(sleepval);
+		slept_time += sleepval;
+		/* exponential backoff, up to 500 ms */
+		sleepval *= 2;
+		if (sleepval > 500)
+			sleepval = 500;
+
 		/* Read a status report from the key */
 		memset(data, 0, sizeof(data));
 		if (!_ykusb_read(yk, REPORT_TYPE_FEATURE, slot, (char *) &data, FEATURE_RPT_SIZE))
 			return 0;
+#ifdef YK_DEBUG
+		_yk_hexdump(data, FEATURE_RPT_SIZE);
+#endif
 
 		if (last_data != NULL)
 			memcpy(last_data, data, sizeof(data));
@@ -508,13 +525,6 @@ int yk_wait_for_key_status(YK_KEY *yk, uint8_t slot, unsigned int flags,
 				break;
 			}
 		}
-
-		Sleep(sleepval);
-		slept_time += sleepval;
-		/* exponential backoff, up to 500 ms */
-		sleepval *= 2;
-		if (sleepval > 500)
-			sleepval = 500;
 	}
 
 	yk_errno = YK_ETIMEOUT;
@@ -713,8 +723,9 @@ uint16_t yk_endian_swap_16(uint16_t x)
 	return x;
 }
 
+#ifdef YK_DEBUG
 /* Private little hexdump function for debugging */
-void _yk_hexdump(void *buffer, int size)
+static void _yk_hexdump(void *buffer, int size)
 {
        unsigned char *p = buffer;
        int i;
@@ -725,3 +736,4 @@ void _yk_hexdump(void *buffer, int size)
       fprintf(stderr, "\n");
       fflush(stderr);
 }
+#endif
